class Participant:
    """! A participant is a device which manages things.

    It can communicate with other participant to synchronise the state of things.
    This class is used to register the things the participant is managing.
    It also maintains the communcation information to contact the participant.
    It is used as a basis for the local participant, but also as a reference to remote participants.
    """
    def __init__(self, ip_address, port):
        """! Create a new participant with the given communcation info
        @param ip_address The IP address of the participant
        @param port The UDP port of the participant
        """

        ## The Ip Address of a participant. When the participant is local, this contains 0.0.0.0
        self.ip_address = ip_address
        ## The port number for UDP communication with the participant. This is 0 for isolated participants.
        self.port = port

        ## he network Id to identify the participant.
        self.network_id =0

        ## The things managed by this participant
        self.things = set()

    def Get(self, thing_id):
        """!  Get the thing with the given properties

        @param thing_id The ID of the thing
        @return The thing if found, None in other cases
        """
        for thing in self.things:
            if thing is not None and thing.id == thing_id:
                return thing
        return None

    def Add(self, thing, check_id = True):
        """! Add a new thing for this participant. 
        @param thing The thing to add
        @param check_id If true, the thing.id is regenerated if it is zero
        """
        if check_id and thing.id == 0:
            thing.id = len(self.things) + 1
            self.things.add(thing)
        else:
            found_thing = self.Get(thing.id)
            if found_thing == None:
                self.things.add(thing)
    
    def Remove(self, thing):
        """! Remove a thing for this participant
        @param thing The thing to remove
        """
        self.things.remove(thing)

    def Update(self, currentTimeMs=0):
        """! Update all things for this participant
        @param The current time in milliseconds (optional)
        """
        for thing in list(self.things):
            if thing is not None:
                thing.Update(currentTimeMs)
